#region
/* Adoor.NET - Object/Relational mapping framework leveraging ADO.NET`s powerful disconnected mode.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 
#endregion

using System;
using System.Collections.Generic;

namespace Adoor.Expression.AST
{
	public class SortExpression
	{
		public ExpressionNode Expr;
		public ScannerToken SortType;
		public SortExpression(ExpressionNode expr, ScannerToken sortType)
		{
			this.Expr = expr;
			this.SortType = sortType;
		}

		public static SortExpression ParseSortExpression(Scanner scanner)
		{
			ExpressionParser ep = new ExpressionParser();
			ExpressionNode expr = ep.ParseNumericPrimary(scanner);
			if (expr == null)
				throw new Exception("name expected");
			ScannerToken sortToken = ScannerToken.Asc;
			if (scanner.Token == ScannerToken.Asc)
			{
				scanner.Scan();
			}			
			else if (scanner.Token == ScannerToken.Desc)
			{
				sortToken = ScannerToken.Desc;
				scanner.Scan();
			}
			return new SortExpression(expr, sortToken);
		}

		// a sort order is an array of sort expressions
		public static SortExpression[] ParseSortOrder(Scanner scanner)
		{
            List<SortExpression> list = new List<SortExpression>();
			do
			{
				scanner.Scan();
				list.Add(SortExpression.ParseSortExpression(scanner));
			} while(scanner.Token == ScannerToken.Comma);
            return list.ToArray();
		}
	}
}
