#region
/* Adoor.NET - Object/Relational mapping framework leveraging ADO.NET`s powerful disconnected mode.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 
#endregion

namespace Adoor.Object.Relational
{
	/// <summary>
	/// Keys may be composite. KeyValue provides a wrapper struct around object[] to enforce strong typing.
	/// A concept related to the key is that of OID : an OID is a string representation of the key.
	/// The OID is easier to manipulate and store outside of the object context.
	/// Keys ans OIDs are unique inside a class (more exactly inside a polymorphic branch,
	/// see IPolymorphicEntityFactory)
	/// You get an entity's OID by calling its ToString() method.
	/// </summary>
	public struct KeyValue
	{
		public KeyValue(params object[] values)
		{
			this.Values = values;
		}

		public object[] Values;

		public override bool Equals(object obj)
		{
			return base.Equals (obj);
		}

		public override int GetHashCode()
		{
			return base.GetHashCode ();
		}

		public override string ToString()
		{
			string[] str = new string[Values.Length];
			for(int i = 0; i < Values.Length; i++)
				str[i] = Values[i].ToString();
			return string.Join(";", str);
		}


	}
}
