
using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Security.Permissions;
using System.Xml;
using Adoor.Object.Domain;
using Adoor.Object.Entity;
using Microsoft.CSharp;
using Microsoft.VisualBasic;
using Adoor.Object.Domain.DomainBuild;

namespace Adoor.Object.Domain.DomainBuild.CodeDom
{
    
    
    internal static class ObjectDomainBuilder
    {
           //protected static log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);


        /// <summary>
        /// Creates a new <see cref="ObjectDomain"/>.
        /// </summary>
        /// <param name="assemblyName">The assembly containing the entity classes for this domain.</param>
        /// <param name="mapFileName">The name of the xml file. If the xml file is embeded as a resource inside an assembly, the name of the assembly followed by the "#" sign should prefix the resource name.</param>
        /// <param name="extendedProperties">A dictionary of extended properties.</param>
        public static void Build(ObjectDomain domain, string assemblyName)
        {
            ObjectDomainXmlMap lMap = new ObjectDomainXmlMap(assemblyName);
            try
            {
                Assembly implementationAssembly = CodeGeneration.CodeGeneratorCodeDom.GetImplementationAssembly(lMap.Assembly, lMap.XmlMap, lMap.NamespaceManager, lMap.Assembly == null);

                ReadMap(domain, implementationAssembly, lMap);
                ReflectiveInitializer initializer = new ReflectiveInitializer(domain);
                initializer.InitInverseReference();
                initializer.InitPropertyAndReference();

            }
            catch (Exception e)
            {
                throw new Exception("Error reading map file stream : " + e.Message, e);
            }


        }
        
        /// <summary>
        /// Creates a new <see cref="ObjectDomain"/>.
        /// </summary>
        /// <param name="assemblyName">The assembly containing the entity classes for this domain.</param>
        /// <param name="mapFileName">The name of the xml file. If the xml file is embeded as a resource inside an assembly, the name of the assembly followed by the "#" sign should prefix the resource name.</param>
        /// <param name="extendedProperties">A dictionary of extended properties.</param>
        public static void Build(ObjectDomain domain, string assemblyName, string mapFileName)
        {
            ObjectDomainXmlMap lMap = new ObjectDomainXmlMap(assemblyName, mapFileName);   
            try
            {
                Assembly implementationAssembly = CodeGeneration.CodeGeneratorCodeDom.GetImplementationAssembly(lMap.Assembly, lMap.XmlMap, lMap.NamespaceManager, lMap.Assembly == null);

                ReadMap(domain, implementationAssembly, lMap);
                ReflectiveInitializer initializer = new ReflectiveInitializer(domain); 
                initializer.InitInverseReference();
                initializer.InitPropertyAndReference();
                
            }
            catch (Exception e)
            {
                throw new Exception("Error reading map file " + mapFileName + ": " + e.Message, e);
            }
        }

        
        private static void ReadMap(ObjectDomain domain, Assembly implementationAssembly, ObjectDomainXmlMap mapp)
        {

            XmlNode xmlMap = mapp.XmlMap;
            XmlNamespaceManager nsmgr = mapp.NamespaceManager;
            bool generate = mapp.Assembly == null;

            //XmlNodeList nodeList = xmlMap.SelectNodes("//*/mapping:entity/..", nsmgr);
            // We need to sort nodes for inheritance issues
            List<XmlNode> alOrderedNodes = XmlMappingReader.SortXmlNode(mapp);

            XmlTypingReader mappReader = new XmlTypingReader(domain, mapp, implementationAssembly, generate);
            mappReader.ReadTypingInfo(alOrderedNodes);

            XmlMappingReader typeReader = new XmlMappingReader(domain, mapp, implementationAssembly, generate);
            typeReader.ReadMappingInfo(alOrderedNodes);

        }



        
    }
}
