
using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Security.Permissions;
using System.Xml;
using Adoor.Object.Domain;
using Adoor.Object.Entity;
using Microsoft.CSharp;
using Microsoft.VisualBasic;


namespace Adoor.Object.Domain.DomainBuild.CodeDom
{
    internal class ObjectDomainXmlMap
    {
        public XmlNamespaceManager NamespaceManager = null;
        public XmlDocument XmlMap = null;
        public Assembly Assembly = null;

        #region contructor(s)


        public ObjectDomainXmlMap(string assemblyName)
        {
            if (assemblyName != null && assemblyName != string.Empty)
                throw new System.Configuration.ConfigurationErrorsException("assembly was not spcified");

            loadAssembly(assemblyName);

            System.IO.MemoryStream stream = new System.IO.MemoryStream(); // where the xml will be written
            //HbmSerializer.Default.Validate = true; // Enable validation (optional)

            // Here, we serialize all decorated classes (but you can also do it class by class)
            Mapping.XmlConfigSerializer.Default.Serialize(stream, this.Assembly);

            Initialize(stream);
        }

        /// <summary>
        /// load from stream
        /// </summary>
        /// <param name="assemblyName"></param>
        /// <param name="stream"></param>
        public ObjectDomainXmlMap(string assemblyName, Stream stream)
        {
            if (assemblyName != null && assemblyName != string.Empty)
                throw new System.Configuration.ConfigurationErrorsException("assembly was not spcified");

            loadAssembly(assemblyName);

            Initialize(stream);
        }

        /// <summary>
        /// load from files
        /// </summary>
        /// <param name="assemblyName"></param>
        /// <param name="mapFileName"></param>
        public ObjectDomainXmlMap(string assemblyName, string mapFileName)
        {
            //Assembly assembly = null;
            if (assemblyName != null && assemblyName != string.Empty)
                loadAssembly(assemblyName);

            int pos = mapFileName.IndexOf("#");
            Stream stream;
            if (pos == -1)
                stream = new FileStream(mapFileName, FileMode.Open, FileAccess.Read);
            else
            {
                string mapAssemblyName = mapFileName.Substring(0, pos);
                string mapResourceName = mapFileName.Substring(pos + 1);
                Assembly mapAssembly = Assembly.Load(mapAssemblyName);
                stream = mapAssembly.GetManifestResourceStream(mapResourceName);
            }

            Initialize(stream);
        }


        private void Initialize(Stream stream)
        {
            this.XmlMap = new XmlDocument();

            loadXmlDocument(stream);

            this.NamespaceManager = new XmlNamespaceManager(new NameTable());
            this.NamespaceManager.AddNamespace("typing", "urn:xmd-typing");
            this.NamespaceManager.AddNamespace("mapping", "urn:xmd-mapping");

        }

        #endregion

        #region Helpers

        /// <summary>
        /// Load an assembly from name and handles Exceptions
        /// </summary>
        /// <param name="assemblyName"></param>
        private void loadAssembly(string assemblyName)
        {
            try
            {
                this.Assembly = Assembly.Load(assemblyName);
            }
            catch (System.IO.FileLoadException ex)
            {
                throw new System.Configuration.ConfigurationErrorsException("Spcified domain assembly '{0}' is not found.", ex);
            }
            catch (BadImageFormatException ex)
            {
                throw new System.Configuration.ConfigurationErrorsException("Spcified domain assembly '{0}' is not a valid image.", ex);
            }
        }

        /// <summary>
        /// Load xml config from stream
        /// </summary>
        /// <param name="stream"></param>
        private void loadXmlDocument(Stream stream)
        {
            if (stream == null)
            {
                if (stream is FileStream)
                    throw new Exception("Mapping info file" + ((FileStream)stream).Name + " not found");
                else
                    throw new Exception("Empty Mapping info stream");
            }

            using (stream)
            {
                this.XmlMap.Load(new XmlTextReader(stream));
            }
            stream.Close();

        }

        #endregion

    }

}
