
using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Security.Permissions;
using System.Xml;
using Adoor.Object.Domain;
using Adoor.Object.Entity;
using Microsoft.CSharp;
using Microsoft.VisualBasic;

namespace Adoor.Object.Domain.DomainBuild.CodeDom
{
    internal class ReflectiveInitializer
    {
        private ObjectDomain _domain = null;

        public ReflectiveInitializer(ObjectDomain domain)
        {
            _domain = domain;
        }

        public void InitInverseReference()
        {
            foreach (EntityData entityData in _domain.EntityInfos)
            {
                foreach (ReferenceData referenceData in entityData.ReferenceInfos.Values)
                {
                    IInverseReference inverseReference = (IInverseReference)referenceData.GetService(typeof(IInverseReference));
                    if (inverseReference != null)
                        inverseReference.InitInverseReference();
                }
            }
        }

        /// <summary>
        /// lookup for 'myPropertyName'Factory_ in implementation type and set it to Entity factory
        /// </summary>
        /// <param name="propertyData"></param>
        private static void initProperty(PropertyData propertyData)
        {
            EntityData sourceData = propertyData.SourceData;
            IEntityFactory sourceFactory = (IEntityFactory)sourceData.GetService(typeof(IEntityFactory));
            if (sourceFactory == null)
                return;
            Type implementationType = sourceFactory.ImplementationType;
            string factoryPropertyName = propertyData.Name + "Factory_";
            FieldInfo fi = implementationType.GetField(factoryPropertyName, BindingFlags.Public | BindingFlags.Static | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
            if (fi == null)
                throw new Exception("property " + factoryPropertyName + " not found in type " + implementationType.Name);

            object service = propertyData.GetService(fi.FieldType);
            if (service == null)
                throw new NullReferenceException("Unable to find " + fi.FieldType.FullName + " in " + implementationType.FullName);
            fi.SetValue(null, service);
        }

        public void InitPropertyAndReference()
        {
            foreach (EntityData entityData in _domain.EntityInfos)
            {
                IService service = (IService)entityData.GetService(typeof(IService));
                if (service != null)
                    service.Initialize();

                foreach (PropertyData propertyData in entityData.PropertyInfos.Values)
                    initProperty(propertyData);

                foreach (ReferenceData referenceData in entityData.ReferenceInfos.Values)
                    initReference(referenceData);
            }
        }

        /// <summary>
        /// lookup for 'TargetRole'Factory_ in implementation type and set it to Entity factory
        /// </summary>
        /// <param name="referenceData"></param>
        private static void initReference(ReferenceData referenceData)
        {
            EntityData sourceData = referenceData.SourceData;
            string propertyName = referenceData.TargetRole + "Factory_";
            IEntityFactory factory = (IEntityFactory)sourceData.GetService(typeof(IEntityFactory));
            if (factory != null && !factory.EntityData.IsAbstract) // otherwise it's an abstract and its descendant will have a factory
            {
                Type implementationType = factory.ImplementationType;
                FieldInfo fi = implementationType.GetField(propertyName, BindingFlags.Public | BindingFlags.Static | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
                if (fi == null)
                    throw new Exception("property " + propertyName + " not found in type " + implementationType.Name);

                object service = referenceData.GetService(fi.FieldType);

                if (service == null)
                    throw new NullReferenceException("Unable to find " + fi.FieldType.FullName + " in " + implementationType.FullName);

                fi.SetValue(null, service);
            }
        }


    }

}
