#region
/* Adoor.NET - Object/Relational mapping framework leveraging ADO.NET`s powerful disconnected mode.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#endregion

#define CSHARP_COMPILER
#define DEBUG_COMPILER

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Reflection;
using Adoor.Object.Entity;
using System.IO;

namespace Adoor.Object.Domain.DomainBuild.CodeGeneration
{

    internal class CodeGeneratorCodeDom //: ICodeGenerator 
    {
        
        //protected static log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        /// <summary>
        /// Generate Codedom from xml configuration and a given type to enhance.
        /// </summary>
        /// <param name="entityType">enhanced type</param>
        /// <param name="metadata">xml configuration data</param>
        /// <param name="nsmgr"></param>
        /// <param name="generate"></param>
        /// <returns></returns>
        public static CodeNamespace GenerateCodeClass(Type entityType, XmlNode metadata, XmlNamespaceManager nsmgr, bool generate)
        {
            string sn = "Runtime";
            string fulltype = metadata.SelectSingleNode("typing:entity/@type", nsmgr).Value;

            if (!generate)
            {
                if (entityType.Namespace!=null)
                    sn = entityType.Namespace + "." + sn;
            }
            else
            {
                fulltype = fulltype.Split(',')[0];
                string[] strs = fulltype.Split('.');
                sn = string.Join(".", strs, 0, strs.Length - 1);
            }
            CodeNamespace Namespace = new CodeNamespace(sn);

            // class Class1Impl: Class1, IEntity
            string entityName = metadata.Name;
            if (!generate)
                entityName = entityType.Name + "_";

            CodeTypeDeclaration Class1 = new CodeTypeDeclaration(entityName);
            Class1.TypeAttributes = TypeAttributes.Public;

            if (!generate)
                Class1.BaseTypes.Add(entityType);
            else
            {
                XmlNode nParent = metadata.SelectSingleNode("typing:entity/@parent", nsmgr);
                if (nParent != null)
                {
                    string parentType = nParent.Value.Split(',')[0];
                    Class1.BaseTypes.Add(parentType);
                }

                nParent = metadata.SelectSingleNode("typing:entity/@abstract", nsmgr);
                if (nParent != null && nParent.Value.ToLower() == "true")
                    Class1.Attributes |= MemberAttributes.Abstract;
            }

            Class1.BaseTypes.Add(typeof(IEntity));
            Namespace.Types.Add(Class1);

            // public Class1Impl(IEntityState state, object[] args)
            CodeConstructor constructor = new CodeConstructor();
            constructor.Parameters.Add(new CodeParameterDeclarationExpression(typeof(IEntityState), "state"));
            constructor.Parameters.Add(new CodeParameterDeclarationExpression(typeof(object[]), "args"));
            constructor.Attributes = MemberAttributes.Public;
            Class1.Members.Add(constructor);

            if (generate)
            {
                CodeConstructor constructor2 = new CodeConstructor();
                constructor2.Attributes = MemberAttributes.Public;
                Class1.Members.Add(constructor2);
            }

            // 	this.state_ = state;
            constructor.Statements.Add(new CodeAssignStatement(
                new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "state_"),
                new CodeArgumentReferenceExpression("state")));

            //	if (args != null)
            //		OnCreate(args);
            //	else
            //		OnMaterialize();
            CodeBinaryOperatorExpression condition = new CodeBinaryOperatorExpression(
                new CodeArgumentReferenceExpression("args"),
                CodeBinaryOperatorType.IdentityInequality,
                new CodePrimitiveExpression(null)
                );

            if (!generate)
            {
                MethodInfo onCreateInfo = entityType.GetMethod("OnCreate", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy, null, new Type[] { typeof(object[]) }, null);
                MethodInfo onMaterializeInfo = entityType.GetMethod("OnMaterialize", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
                if (onCreateInfo != null || onMaterializeInfo != null)
                {

                    CodeExpressionStatement onCreate = new CodeExpressionStatement(new CodeMethodInvokeExpression(new CodeThisReferenceExpression(), "OnCreate", new CodeExpression[] { new CodeArgumentReferenceExpression("args") }));
                    CodeStatement[] trueStatements = onCreateInfo == null ? new CodeStatement[] { } : new CodeStatement[] { onCreate };

                    CodeExpressionStatement onMaterialize = new CodeExpressionStatement(new CodeMethodInvokeExpression(new CodeThisReferenceExpression(), "OnMaterialize", new CodeExpression[] { }));
                    CodeStatement[] falseStatements = onMaterializeInfo == null ? new CodeStatement[] { } : new CodeStatement[] { onMaterialize };
                    constructor.Statements.Add(new CodeConditionStatement(condition, trueStatements, falseStatements));
                }
            }

            // IEntityState IEntity.State
            // {
            //	get { return this.state_; }
            // }
            CodeMemberProperty entityState = new CodeMemberProperty();
            entityState.Attributes = MemberAttributes.ScopeMask;
            entityState.Type = new CodeTypeReference(typeof(IEntityState));
#if CSHARP_COMPILER
            entityState.Name = typeof(IEntity).FullName + ".State";
#else
            entityState.Name = "State";
            entityState.ImplementationTypes.Add(typeof(IEntity));
#endif

            entityState.GetStatements.Add(new CodeMethodReturnStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "state_")));
            Class1.Members.Add(entityState);

            // public override string ToString()
            // {
            //		return state_.Oid;
            // }
            CodeMemberMethod toString = new CodeMemberMethod();
            toString.Attributes = MemberAttributes.Public | MemberAttributes.Override;
            toString.ReturnType = new CodeTypeReference(typeof(string));
            toString.Name = "ToString";
            toString.Statements.Add(
                new CodeMethodReturnStatement(
                new CodeFieldReferenceExpression(
                new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "state_"),
                "Oid")));
            Class1.Members.Add(toString);

            //	void IEntity.OnValidate()
            //  {
            //  }
            if (!generate)
            {
                MethodInfo onValidateInfo = entityType.GetMethod("OnValidate", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
#if CSHARP_COMPILER
                if (onValidateInfo == null)
                {
                    CodeMemberMethod onValidate = new CodeMemberMethod();
                    onValidate.Attributes = MemberAttributes.ScopeMask;
                    onValidate.Name = typeof(IEntity).FullName + ".OnValidate";
                    Class1.Members.Add(onValidate);
                }
            }
            else
            {
                CodeMemberMethod onValidate = new CodeMemberMethod();
                onValidate.Attributes = MemberAttributes.ScopeMask;
                onValidate.Name = typeof(IEntity).FullName + ".OnValidate";
                Class1.Members.Add(onValidate);
            }
#else
                CodeMemberMethod onValidate = new CodeMemberMethod();
                onValidate.Attributes = MemberAttributes.ScopeMask;
                onValidate.Name = "OnValidate";
                onValidate.ImplementationTypes.Add(typeof(IEntity));
                if (onValidateInfo != null)
                {
                    onValidate.Statements.Add(new CodeMethodReferenceExpression(new CodeBaseReferenceExpression(), "OnValidate"));
                }
                Class1.Members.Add(onValidate);
#endif

                foreach (XmlNode propNode in metadata.SelectNodes("*/mapping:property", nsmgr))
                {
                    string propertyName = propNode.ParentNode.LocalName;
                    PropertyInfo propertyInfo = null;

                    if (!generate)
                    {
                        XmlNode storageNode = propNode.ParentNode.SelectSingleNode("typing:property/@storage", nsmgr);
                        if (storageNode != null)
                        {
                            propertyInfo = entityType.GetProperty(storageNode.Value, BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
                            if (propertyInfo == null)
                                throw new Exception("property " + storageNode.Value + " not found in class " + entityType.FullName);
                        }
                        else
                            // first try with the protected case insensitive m-prefixed version
                            propertyInfo = entityType.GetProperty("m_" + propertyName, BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy | BindingFlags.IgnoreCase);
                        if (propertyInfo == null)
                        {
                            // second try with the protected case insensitive version
                            propertyInfo = entityType.GetProperty(propertyName, BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy | BindingFlags.IgnoreCase);
                        }
                        if (propertyInfo == null)
                        {
                            // third try with the public case sensitive version
                            propertyInfo = entityType.GetProperty(propertyName, BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
                        }
                        if (propertyInfo == null)
                            throw new Exception("property " + propertyName + " not found for type " + entityType.Name);

                        if (!(propertyInfo.PropertyType.IsValueType || propertyInfo.PropertyType == typeof(string) || propertyInfo.PropertyType == typeof(byte[])))
                            throw new Exception("invalid propertyInfo");
                    }
                    else
                    {
                        XmlNode inherited = propNode.ParentNode.SelectSingleNode("typing:property/@inherited", nsmgr);
                        if (inherited != null && inherited.Value.ToLower() == "true")
                            continue;
                    }

                    XmlNode aliasNode = propNode.SelectSingleNode("@alias");
                    string alias = aliasNode != null ? aliasNode.Value : propNode.ParentNode.LocalName;

                    Type propertyType = null;
                    string sType = propNode.ParentNode.SelectSingleNode("typing:property/@type", nsmgr).Value;
                    if (!generate)
                        propertyType = Type.GetType(sType, true);
                    else
                    {
                        propertyType = Type.GetType(sType, false);
                    }

                    //		public static PropertyData NameFactory_;
                    CodeMemberField nameFactory = new CodeMemberField(typeof(IPropertyWriter), propertyName + "Factory_");
                    nameFactory.Attributes = MemberAttributes.Public | MemberAttributes.Static;

                    // public override type Name
                    // {
                    //   get { return NameFactory_.GetType(this); }
                    //   set { NameFactory_.SetType(this, value); }
                    // }
                    CodeMemberProperty property = new CodeMemberProperty();
                    if (!generate)
                    {
                        property.Attributes = MemberAttributes.Override;
                        property.Type = new CodeTypeReference(propertyInfo.PropertyType);
                    }
                    else
                    {
                        string[] strs = sType.Split(',');
                        if (propertyType == null)
                            property.Type = new CodeTypeReference(typeof(int));
                        else
                            property.Type = new CodeTypeReference(strs[0]);
                        property.Attributes = MemberAttributes.Public;
                    }

                    if (!generate)
                    {
                        property.Name = propertyInfo.Name;
                        string propertyTypeName = propertyInfo.PropertyType.Name.Replace("[]", "Array");
                        MethodInfo getMethodInfo = propertyInfo.GetGetMethod(true);
                        if (getMethodInfo != null)
                        {
                            if (!getMethodInfo.IsAbstract)
                                throw new Exception("property " + propertyName + " of type " + entityType.FullName + " should be abstract");
                            if (!propertyInfo.PropertyType.IsAssignableFrom(propertyType))
                                throw new Exception("type mismatch for property " + propertyName);
                            if (getMethodInfo.IsPublic)
                                property.Attributes |= MemberAttributes.Public;
                            else
                                property.Attributes |= MemberAttributes.Family;
                            if (propertyInfo.PropertyType.IsEnum)
                            {
                                property.GetStatements.Add(
                                    new CodeMethodReturnStatement(
                                    new CodeCastExpression(propertyInfo.PropertyType,
                                    new CodeMethodInvokeExpression(
                                    new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(Class1.Name), propertyName + "Factory_"),
                                    "GetInt32",
                                    new CodeExpression[] { new CodeThisReferenceExpression() }))));
                            }
                            else
                            {
                                property.GetStatements.Add(
                                    new CodeMethodReturnStatement(
                                    new CodeMethodInvokeExpression(
                                    new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(Class1.Name), propertyName + "Factory_"),
                                    "Get" + propertyTypeName,
                                    new CodeExpression[] { new CodeThisReferenceExpression() })));
                            }
                        }
                        MethodInfo setMethodInfo = propertyInfo.GetSetMethod(true);
                        if (setMethodInfo != null)
                        {
                            if (!setMethodInfo.IsAbstract)
                                throw new Exception("property " + propertyName + " of type " + entityType.FullName + " should be abstract");
                            if (!propertyType.IsAssignableFrom(propertyInfo.PropertyType))
                                throw new Exception("type mismatch for property " + propertyName);
                            if (setMethodInfo.IsPublic)
                                property.Attributes |= MemberAttributes.Public;
                            else
                                property.Attributes |= MemberAttributes.Family;
                            if (propertyInfo.PropertyType.IsEnum)
                            {
                                property.SetStatements.Add(
                                    new CodeExpressionStatement(
                                    new CodeMethodInvokeExpression(
                                    new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(Class1.Name), propertyName + "Factory_"),
                                    "SetInt32",
                                    new CodeExpression[] { new CodeThisReferenceExpression(), new CodeCastExpression(typeof(int), new CodePropertySetValueReferenceExpression()) })));
                            }
                            else
                            {
                                property.SetStatements.Add(
                                    new CodeExpressionStatement(
                                    new CodeMethodInvokeExpression(
                                    new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(Class1.Name), propertyName + "Factory_"),
                                    "Set" + propertyTypeName,
                                    new CodeExpression[] { new CodeThisReferenceExpression(), new CodePropertySetValueReferenceExpression() })));
                            }
                        }
                    }
                    else
                    {
                        property.Name = propertyName;
                        string[] strs = sType.Split(',');
                        string propertyTypeName = strs[0].Replace("[]", "Array");
                        string[] strs2 = propertyTypeName.Split('.');


                        if (propertyType == null || propertyType.IsEnum)
                        {
                            property.GetStatements.Add(
                                new CodeMethodReturnStatement(
                                new CodeMethodInvokeExpression(
                                new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(fulltype), propertyName + "Factory_"),
                                "GetInt32",
                                new CodeExpression[] { new CodeThisReferenceExpression() })));
                        }
                        else
                        {

                            property.GetStatements.Add(
                                new CodeMethodReturnStatement(
                                new CodeMethodInvokeExpression(
                                new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(fulltype), propertyName + "Factory_"),
                                "Get" + strs2[strs2.Length - 1],
                                new CodeExpression[] { new CodeThisReferenceExpression() })));
                        }


                        if (propertyType == null || propertyType.IsEnum)
                        {
                            property.SetStatements.Add(
                                new CodeExpressionStatement(
                                new CodeMethodInvokeExpression(
                                new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(fulltype), propertyName + "Factory_"),
                                "SetInt32",
                                new CodeExpression[] { new CodeThisReferenceExpression(), new CodeCastExpression(typeof(int), new CodePropertySetValueReferenceExpression()) })));
                        }
                        else
                        {
                            property.SetStatements.Add(
                                new CodeExpressionStatement(
                                new CodeMethodInvokeExpression(
                                new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(fulltype), propertyName + "Factory_"),
                                "Set" + strs2[strs2.Length - 1],
                                new CodeExpression[] { new CodeThisReferenceExpression(), new CodePropertySetValueReferenceExpression() })));
                        }

                    }
                    if (property.GetStatements.Count != 0 || property.SetStatements.Count != 0)
                    {
                        Class1.Members.Add(nameFactory);
                        Class1.Members.Add(property);
                    }
                }


                foreach (XmlNode propNode in metadata.SelectNodes("*/mapping:reference", nsmgr))
                {
                    string propertyName = propNode.ParentNode.LocalName;
                    PropertyInfo propertyInfo = null;

                    if (!generate)
                    {
                        XmlNode storageNode = propNode.ParentNode.SelectSingleNode("typing:property/@storage", nsmgr);
                        if (storageNode != null)
                        {
                            propertyInfo = entityType.GetProperty(storageNode.Value, BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
                            if (propertyInfo == null)
                                throw new Exception("property " + storageNode.Value + " not found in class " + entityType.FullName);
                        }
                        else
                            // first try with the protected case insensitive m-prefixed version
                            propertyInfo = entityType.GetProperty("m_" + propertyName, BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy | BindingFlags.IgnoreCase);
                        if (propertyInfo == null)
                        {
                            // second try with the protected case insensitive version
                            propertyInfo = entityType.GetProperty(propertyName, BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy | BindingFlags.IgnoreCase);
                        }
                        if (propertyInfo == null)
                        {
                            // third try with the public case sensitive version
                            propertyInfo = entityType.GetProperty(propertyName, BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
                        }
                        if (propertyInfo == null)
                            throw new Exception("property " + entityType.Name + "." + propertyName + " not found");
                        if (propertyInfo.PropertyType.IsValueType || propertyInfo.PropertyType == typeof(string) || propertyInfo.PropertyType == typeof(byte[]))
                            throw new Exception("property " + entityType.Name + "." + propertyName + " is not a reference");
                    }
                    else
                    {
                        XmlNode inherited = propNode.ParentNode.SelectSingleNode("typing:reference/@inherited", nsmgr);
                        if (inherited != null && inherited.Value.ToLower() == "true")
                            continue;
                    }
                    XmlNode aliasNode = propNode.SelectSingleNode("@alias");
                    string alias = aliasNode != null ? aliasNode.Value : propNode.ParentNode.LocalName;
                    XmlNode multiplicityNode = propNode.ParentNode.SelectSingleNode("typing:reference/@multiplicity", nsmgr);
                    if (multiplicityNode == null)
                        throw new Exception("attribute multiplicity not found for property " + propertyName + " of type " + entityType.Name);
                    if (multiplicityNode.Value == "1")
                    {
                        string sType = propNode.ParentNode.SelectSingleNode("typing:reference/@type", nsmgr).Value;
                        string type = sType.Split(',')[0];

                        //		public static ReferenceData NameFactory_;
                        CodeMemberField nameFactory = new CodeMemberField(
                            new CodeTypeReference("Adoor.Object.Entity.ISingleReferenceWriter"
                                , new CodeTypeReference(type))
                                , propertyName + "Factory_");
                        nameFactory.Attributes = MemberAttributes.Public | MemberAttributes.Static;


                        //		public override type Name
                        //		{
                        //			get { return (type)NameFactory_.GetValue(this); }
                        //			set { NameFactory_.SetValue(this, (IEntity)value); }
                        //		}
                        CodeMemberProperty property = new CodeMemberProperty();
                        property.Attributes = MemberAttributes.Override;
                        if (!generate)
                            property.Type = new CodeTypeReference(propertyInfo.PropertyType);
                        else
                        {
                            string[] strs = sType.Split(',');
                            property.Type = new CodeTypeReference(strs[0]);
                        }

                        if (!generate)
                            property.Name = propertyInfo.Name;
                        else
                            property.Name = propertyName;

                        if (!generate)
                        {
                            MethodInfo getMethodInfo = propertyInfo.GetGetMethod(true);
                            if (getMethodInfo != null)
                            {
                                if (!getMethodInfo.IsAbstract)
                                    throw new Exception("property " + propertyName + " of type " + entityType.FullName + " should be abstract");
                                if (getMethodInfo.IsPublic)
                                    property.Attributes |= MemberAttributes.Public;
                                else
                                    property.Attributes |= MemberAttributes.Family;
                                CodeMethodInvokeExpression cmie = new CodeMethodInvokeExpression(
                                    new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(Class1.Name), propertyName + "Factory_"),
                                    "GetValue",
                                    new CodeExpression[] { new CodeThisReferenceExpression() });
                                //cmie.Method.TypeArguments.Add(property.Type);
                                property.GetStatements.Add(new CodeMethodReturnStatement(cmie));
                            }
                            MethodInfo setMethodInfo = propertyInfo.GetSetMethod(true);
                            if (setMethodInfo != null)
                            {
                                if (!setMethodInfo.IsAbstract)
                                    throw new Exception("property " + propertyName + " of type " + entityType.FullName + " should be abstract");
                                if (setMethodInfo.IsPublic)
                                    property.Attributes |= MemberAttributes.Public;
                                else
                                    property.Attributes |= MemberAttributes.Family;
                                property.SetStatements.Add(
                                    new CodeExpressionStatement(
                                    new CodeMethodInvokeExpression(
                                    new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(Class1.Name), propertyName + "Factory_"),
                                    "SetValue",
                                    new CodeExpression[] { new CodeThisReferenceExpression(), new CodeCastExpression(typeof(IEntity), new CodePropertySetValueReferenceExpression()) })));
                            }
                        }
                        else
                        {
                            string[] strs = sType.Split(',');
                            property.Attributes = MemberAttributes.Public;
                            CodeMethodInvokeExpression cmie = new CodeMethodInvokeExpression(
                                new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(fulltype), propertyName + "Factory_"),
                                "GetValue",
                                new CodeExpression[] { new CodeThisReferenceExpression() });
                            //cmie.Method.TypeArguments.Add(property.Type);

                            property.GetStatements.Add(new CodeMethodReturnStatement(cmie));

                            property.Attributes = MemberAttributes.Public;
                            property.SetStatements.Add(
                                new CodeExpressionStatement(
                                new CodeMethodInvokeExpression(
                                new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(fulltype), propertyName + "Factory_"),
                                "SetValue",
                                new CodeExpression[] { new CodeThisReferenceExpression(), new CodeCastExpression(typeof(IEntity), new CodePropertySetValueReferenceExpression()) })));

                        }
                        if (property.GetStatements.Count != 0 || property.SetStatements.Count != 0)
                        {
                            Class1.Members.Add(nameFactory);
                            Class1.Members.Add(property);
                        }
                    }
                    else if (multiplicityNode.Value == "*")
                    {
                        string sType = propNode.ParentNode.SelectSingleNode("typing:reference/@type", nsmgr).Value;
                        string type = sType.Split(',')[0];
                        //		public static IListFactory NameFactory_;
                        CodeMemberField nameFactory = new CodeMemberField(
                           new CodeTypeReference("Adoor.Object.Entity.IReferenceWriter"
                               , new CodeTypeReference(type))
                               , propertyName + "Factory_");
                        nameFactory.Attributes = MemberAttributes.Public | MemberAttributes.Static;


                        //		public override IList Name
                        //		{
                        //			get { return NameFactory_.Collect(this); }
                        //		}
                        CodeMemberProperty property = new CodeMemberProperty();
                        if (!generate)
                        {
                            property.Attributes = MemberAttributes.Override;
                            property.Type = new CodeTypeReference(propertyInfo.PropertyType);
                        }
                        else
                        {
                            string[] strs = sType.Split(',');
                            property.Attributes = MemberAttributes.Public;
                            property.Type = new CodeTypeReference(strs[0]);
                        }

                        if (!generate)
                        {
                            property.Name = propertyInfo.Name;
                            MethodInfo getMethodInfo = propertyInfo.GetGetMethod(true);
                            if (getMethodInfo != null && getMethodInfo.IsAbstract)
                            {
                                if (getMethodInfo.IsPublic)
                                    property.Attributes |= MemberAttributes.Public;
                                else
                                    property.Attributes |= MemberAttributes.Family;

                                CodeMethodInvokeExpression cmie = new CodeMethodInvokeExpression(
                                    new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(Class1.Name), propertyName + "Factory_"),
                                    "Collect",
                                    new CodeExpression[] { new CodeThisReferenceExpression() });
                                //cmie.Method.TypeArguments.Add(type);
                                property.GetStatements.Add(new CodeMethodReturnStatement(cmie));
                                Class1.Members.Add(nameFactory);
                                Class1.Members.Add(property);
                            }
                        }
                        else
                        {
                            property.Name = propertyName;
                            property.Attributes = MemberAttributes.Public;

                            property.Type = new CodeTypeReference("System.Collections.Generic.IList",
                                new CodeTypeReference(type));

                            CodeMethodInvokeExpression cmie = new CodeMethodInvokeExpression(
                                    new CodeFieldReferenceExpression(new CodeTypeReferenceExpression(Class1.Name), propertyName + "Factory_"),
                                    "Collect",
                                    new CodeExpression[] { new CodeThisReferenceExpression() });
                            //cmie.Method.TypeArguments.Add(type);
                            property.GetStatements.Add(new CodeMethodReturnStatement(cmie));
                            Class1.Members.Add(nameFactory);
                            Class1.Members.Add(property);
                        }
                    }
                    else
                        throw new Exception("bad multiplicity value for property " + propertyName + " of type " + entityType.FullName);
                }

                // private IEntityState state_;
                CodeMemberField state = new CodeMemberField(typeof(IEntityState), "state_");
                //state.Attributes = MemberAttributes.Family;
                state.Attributes = MemberAttributes.Private;
                Class1.Members.Add(state);

                return Namespace;
            }

        /// <summary>
        /// Compile a generated assembly from CodeDom
        /// </summary>
        /// <param name="baseAssembly">base domain assembly</param>
        /// <param name="compunit">CodeDom unit to compile</param>
        /// <returns></returns>
        public static Assembly CompileUnit(Assembly baseAssembly, CodeCompileUnit compunit)
        {
            Adoor.AdoorLoger.Debug("Compile domain : " + baseAssembly != null ? baseAssembly.GetName().Name : " new assembly");

            CompilerParameters compparams = new CompilerParameters(
                new string[] { Assembly.Load("Adoor.Relational").Location
                    , Assembly.Load("Adoor.Object").Location });

            if (baseAssembly != null)
            {
                foreach (AssemblyName a in baseAssembly.GetReferencedAssemblies())
                {
                    Assembly assembly = Assembly.Load(a.FullName);
                    foreach (Module m in assembly.GetModules())
                        compparams.ReferencedAssemblies.Add(m.FullyQualifiedName);
                }

                foreach (Module m in baseAssembly.GetModules())
                    compparams.ReferencedAssemblies.Add(m.FullyQualifiedName);
            }
            bool debugCompiler = AppSettingsPath.DebugCompiler;

            string providerString = "CSharp";
            string newProvider = AppSettingsPath.CompilerProviderType;
            bool debugInfo = AppSettingsPath.IncludeDebugInfo;

            if (!string.IsNullOrEmpty(newProvider))
                providerString = newProvider;

            CodeDomProvider provider = CodeDomProvider.CreateProvider(providerString);

            compparams.GenerateInMemory = false;

            string output = GeneratedAssemblyPath(baseAssembly);
            compparams.OutputAssembly = output;

            compparams.IncludeDebugInformation = debugInfo;


            if (debugCompiler)
            {
                string file = GeneratedAssemblyDirectory(baseAssembly);
                if (!Directory.Exists(file))
                    Directory.CreateDirectory(file);

                if (baseAssembly != null)
                {
                    file = Path.Combine(file, baseAssembly.GetName().Name);

                    using (FileStream fs = new FileStream(file + "_.cs", FileMode.OpenOrCreate, FileAccess.Write, FileShare.None))
                    {
                        using (TextWriter writer = new StreamWriter(fs))
                        {
                            provider.GenerateCodeFromCompileUnit(compunit, writer, null);
                        }
                    }
                }
            }


            CompilerResults compresult = provider.CompileAssemblyFromDom(compparams, compunit);
            foreach (CompilerError error in compresult.Errors)
                Adoor.AdoorLoger.Debug(error.Line + " : " + error.ErrorText);
            foreach (CompilerError error in compresult.Errors)
                throw new Exception(error.Line + " : " + error.ErrorText);

            return compresult.CompiledAssembly;
        }

        private static string GeneratedAssemblyDirectory(Assembly baseAssembly)
        {

            //default to a temp directory
            string outPath = Path.GetTempPath();
            string output = AppSettingsPath.CompilerOutputAssemblyFolder;
            if (!string.IsNullOrEmpty(output))
                outPath = output;

            return outPath;

        }

        /// <summary>
        /// Retreive CodeBase for generated assembly
        /// </summary>
        /// <remarks>depending on number of computation, it may be 
        /// interesting to cache this data</remarks>
        /// <param name="baseAssembly">domain Assembly beeing instrumented</param>
        /// <returns></returns>
        public static string GeneratedAssemblyPath(Assembly baseAssembly)
        {
            if (baseAssembly == null)
                return string.Empty;
            //default to a temp directory
            string outPath = Path.GetTempPath();
            string assemblyFileName = string.Empty;
            assemblyFileName = baseAssembly.GetName().Name + "_.dll";
                outPath = Path.Combine(outPath, assemblyFileName);

            //if specified use specific folder
            string output = AppSettingsPath.CompilerOutputAssemblyFolder;
            if (!string.IsNullOrEmpty(output) && baseAssembly != null)
                outPath = System.IO.Path.Combine(output, assemblyFileName);

            return outPath;
        }

        /// <summary>
        /// Load a generated assembly (either a JIT generated assembly or an older one)
        /// </summary>
        /// <param name="baseAssembly"></param>
        /// <param name="xmlMap"></param>
        /// <param name="nsmgr"></param>
        /// <param name="generate"></param>
        /// <returns></returns>
        public static Assembly GetImplementationAssembly(Assembly baseAssembly, XmlNode xmlMap, XmlNamespaceManager nsmgr, bool generate)
        {
            if (CanReuseAssembly(baseAssembly))
            {
                return Assembly.LoadFrom(GeneratedAssemblyPath(baseAssembly));
            }
            else
            {
                CodeCompileUnit compunit = new CodeCompileUnit();
                if (!generate)
                {
                    foreach (XmlNode metadata in xmlMap.SelectNodes("//*[not(typing:entity/@abstract='true')]/mapping:entity/..", nsmgr))
                    {
                        Type entityType = null;
                        if (!generate)
                            entityType = getEntityType(metadata, nsmgr);

                        compunit.Namespaces.Add(GenerateCodeClass(entityType, metadata, nsmgr, generate));
                    }
                }
                else
                {
                    foreach (XmlNode metadata in xmlMap.SelectNodes("//*/mapping:entity/..", nsmgr))
                    {
                        Type entityType = null;

                        compunit.Namespaces.Add(GenerateCodeClass(entityType, metadata, nsmgr, generate));
                    }
                }
                return CompileUnit(baseAssembly, compunit);
            }
        }

        /// <summary>
        /// Test Whether Adoor should reuse a previously generated assembly
        /// </summary>
        /// <param name="baseAssembly">Domain base assembly</param>
        /// <returns></returns>
        private static bool CanReuseAssembly(Assembly baseAssembly)
        {
            if (AppSettingsPath.DoNotRegenerateAssemblies)
                return DoesAssemblyExist(baseAssembly);

            return false;
        }

        /// <summary>
        /// Test whether an previously generated assembly exists
        /// </summary>
        /// <param name="baseAssembly">Domain base assembly</param>
        /// <returns></returns>
        private static bool DoesAssemblyExist(Assembly baseAssembly)
        {
            return File.Exists(GeneratedAssemblyPath(baseAssembly));
        }

        [System.Security.Permissions.ReflectionPermission(System.Security.Permissions.SecurityAction.Demand)]
        public static Type getEntityType(XmlNode metadata, XmlNamespaceManager nsmgr)
        {
            XmlNode typeNode = metadata.SelectSingleNode("typing:entity/@type", nsmgr);
            if (typeNode == null)
                throw new Exception("type not found for node " + metadata.Name);
            return Type.GetType(typeNode.Value, true);
        }


    }


}
