#region
/* Adoor.NET - Object/Relational mapping framework leveraging ADO.NET`s powerful disconnected mode.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 
#endregion

using Adoor.Object.Disconnected;

namespace Adoor.Object.Entity
{
	/// <summary>
	///	<see cref="IEntity"/> is an interface supported by the implementation classes of entity classes.
	/// </summary>
	/// <remarks>It is used internally to give access to the <see cref="State"/> property.</remarks>
	public interface IEntity
	{
		IEntityState State { get; }
		void OnValidate();
	}

	/// <summary>
	/// The modification status of an entity.
	/// </summary>
	public enum EntityStatus { Unchanged, Modified, Added, Deleted }

	/// <summary>
	/// Interface giving access to the state of an entity.
	/// </summary>
	public interface IEntityState
	{
		IObjectStorage Storage { get; }
		IEntityFactory EntityFactory { get; }
		EntityStatus Status { get; }
		string Oid { get; }
		void Touch();
		void Delete();

		/* disconnected aspect */
		bool IsReferenceComplete(string roleName, ref PreloadFlags flags); // TODO: in IReferenceFactory ?
		void CompleteReference(string roleName, PreloadFlags flags); // TODO: in IReferenceFactory ?
	}
}