#region
/* Adoor.NET - Object/Relational mapping framework leveraging ADO.NET`s powerful disconnected mode.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 
#endregion

using System.Collections.Generic;
using Adoor.Object.Disconnected;
using Adoor.Object.Domain;
using Adoor.Object.Query;

namespace Adoor.Object.Entity
{
    public interface IReferenceReader
    {
        ReferenceData ReferenceData { get; }

        void InitInverseReferenceReader<TInverse>(ReferenceData inverseReferenceData);

        System.Collections.IEnumerator EnumTargetKeys(IEntity entity);  // TODO: EnumTargetOids ? or in IReferenceMappingStrategy ?

        // TODO: in disconnected
        void FetchReference(ObjectContext context, ObjectPath path, object args, string root, string alias, bool isGloballyLoaded);
        void FetchTargetEntities(IEntity entity);

    }

	/// <summary>
	/// An interface supported by reference mappers for the reading of reference values.
	/// </summary>
	public interface IReferenceReader<TReference> : IReferenceReader
	{
		IList<TReference> Collect(IEntity entity);
    }

	/// <summary>
	/// An interface supported by single reference mappers for the reading of reference values.
	/// </summary>
    public interface ISingleReferenceReader<TReference> : IReferenceReader<TReference>
	{
		TReference GetValue(IEntity sourceEntity);
	}
}