using System;

namespace Adoor.Metadata 
{
	public class XmdEnvironment : IMessageProvider, IProgress
	{
		public string TableName(string s1, string s2)
		{
			if(s1.CompareTo(s2) > 0)
				return string.Concat(s1, "_", s2);
			else
				return string.Concat(s2, "_", s1);
			//return (s1.Length <= 8 ? s1 : s1.Substring(0, 8)) + "_" + (s2.Length <= 8 ? s2 : s2.Substring(0, 8));
		}

		public string FullTypeName(string s)
		{
			int i = s.IndexOf(",");
			if (i == -1)
				return s;
			return s.Substring(0, i);
		}

		public string TypeName(string s)
		{
			string result = FullTypeName(s);
			int i = result.LastIndexOf(".");
			if (i == -1)
				return result;
			return result.Substring(i + 1);
		}

		public string NamespaceName(string s)
		{
			string result = FullTypeName(s);
			int i = result.LastIndexOf(".");
			if (i == -1)
				return result;
			return result.Substring(0, i);
		}

		public string FilePath(string s)
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder(s.Length);
			string[] strs = s.Split(',');
			if(strs.Length == 0)
				return s.Replace(".","\\");
			else
			{
				string[] strs1 = strs[0].Trim().Split('.');
				string[] strs2 = strs[1].Trim().Split('.');

				int i = 0;
				for(; i < strs1.Length && i < strs2.Length; i++)
				{
					string a = strs1[i].Trim();
					string b = strs2[i].Trim();
					
					if(!a.Equals(b))
						break;
				}

				for(int j = 0; j < strs1.Length - 1; j++)
				{
					if(j != 0)
					{
						if(j < i)
							sb.Append(".");
						else
							sb.Append("\\");
					}					
	
					sb.Append(strs1[j]);
				}

			}

			return sb.ToString();
		}

		public string DbType(string s)
		{
			// TODO: use a hashtable(Type->string)
			Type type = Type.GetType(s);
			return type.Name;
			/*
			int <-> SqlInt32
			bit <-> SqlBoolean
			smallint <-> SqlInt16
			datetime <-> SqlDateTime
			ntext <-> SqlString
			nvarchar <-> SqlString
			image <-> SqlBinary
			bigint <-> SqlInt64
			binary <-> SqlBinary
			char <-> SqlString
			decimal <-> SqlDecimal
			float <-> SqlDouble
			money <-> SqlMoney
			nchar <-> SqlString
			real <-> SqlSingle
			smalldatetime <-> SqlDateTime
			smallmoney <-> SqlMoney
			text <-> SqlString
			timestamp <-> SqlBinary
			tinyint <-> SqlByte
			uniqueidentifier <-> SqlGuid
			varbinary <-> SqlBinary
			varchar <-> SqlString
			variant <-> Object
			*/
		}

		public string PublicName(string s)
		{
			return s[0].ToString().ToUpper() + s.Substring(1);
		}

		public string ProtectedName(string s)
		{
			return s[0].ToString().ToLower() + s.Substring(1);
		}

		public string QName(string s)
		{
			string result = "";
			for (int i = 0; i < s.Length; i++)
				if ("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.".IndexOf(s[i]) != -1)
					result += s[i];
			//				else
			//					result += string.Format("{0:x}", s[i]);
			return result;
		}

		public string SelectNodes(object node, string xpath)
		{
			//return node
			//			return node.
			//				.SelectNodes(xpath);
			return "toto";
		}

		public void Log(string message, string level)
		{
			int iLevel = 0;
			try
			{
				iLevel = int.Parse(level);
			}
			catch
			{
			}
			OnLogMessage(message, iLevel);
		}

		public void SetProgressTotal(int total)
		{
			progressCnt = 0;
			progressTotal = total;
            OnProgress();
		}

		private int progressCnt;
		private int progressTotal;
		public void UpdateProgress()
		{			
			progressCnt++;
			if(progressCnt >= 0 && progressCnt < progressTotal)
				OnProgress();
		}

		protected virtual void OnProgress()
		{
			if(Progress != null)
				Progress(this, new ProgressEventArgs(progressCnt, progressTotal));
		}
		
		protected virtual void OnLogMessage(string message, int level)
		{
			if(LogMessage != null)
				LogMessage(this, new LogMessageEventArgs(message, level));
		}
		
		public event LogMessageEventHandler LogMessage;
		public event ProgressEventHandler Progress;		
	
	}
}
