using System;
using System.Xml;

namespace Adoor.Metadata
{
	public class XmlMultiFileWriter: XmlWriterDecorator
	{
		private IXmlWriterProvider writerProvider;

		public override XmlWriter Principal
		{
			get
			{
				if (base.Principal == null)
					base.Principal = writerProvider.GetWriter(null);
				return base.Principal;
			}

			set
			{
				if (base.Principal != null)
					base.Principal.Close();
				base.Principal = value;
			}
		}
	
		public XmlMultiFileWriter(IXmlWriterProvider writerProvider) :
			base(null)
		{
			this.writerProvider = writerProvider;
		}

		public override void WriteProcessingInstruction(string name, string text)
		{
			if (name == "file") 
			{
                this.Principal = this.writerProvider.GetWriter(text.Trim());
			}
			else
				base.WriteProcessingInstruction(name, text);
		}

        public override void Close()
        {
            base.Close();
            this.Principal = null;
        }

        protected override void Dispose(bool disposing)
        {
            if (this.Principal != null)
                this.Principal.Close();

            base.Dispose(disposing);
        }
	}
}