using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using System.Diagnostics;

using System.Data;


using Adoor.Domain.Tiers;



namespace Adoor.Nunit.Basic
{
    [TestFixture()]
    public class BasicCRUD : BaseTest
    {
        [Test()]
        public void Test1()
        {

        }

        public Adoor.Object.Disconnected.ObjectContext Context;

        [SetUp]
        public void Setup()
        {
            _OIDPerson = string.Empty;

            //log4net.Repository.ILoggerRepository rep = log4net.LogManager.GetRepository();
            //log4net.Config.BasicConfigurator.Configure(
            //                                        new log4net.Appender.ConsoleAppender(
            //                                                        new log4net.Layout.PatternLayout("%d [%t] %-5p %c [%x] &lt;%X{abc}&gt; - %m%n")));

            Context = CreateContext();
            Trace.Listeners.Add(new TextWriterTraceListener(Console.Out, "Console"));
        }

        [TearDown]
        public void TearDown()
        {
            Trace.Listeners.Remove("Console");
            _OIDPerson = string.Empty;
        }

        private string _OIDPerson = string.Empty;
 
        [Test(Description = "Create a personne with adresse, Coordonnee and Lieu")]
        public void TestObjectdependencies()
        {
            Personne result = CreatePersonne(Context);
            result.Adresse = CreateAdresse(Context);
            result.AdresseDeces = CreateAdresse(Context);
            result.AdresseNaissance = CreateAdresse(Context);  
            result.AdresseSuccession = CreateAdresse(Context);  
            result.Coordonnee = CreateCoordonnee(Context);
            result.CoordonneeSuccession = CreateCoordonnee(Context);
            Context.Update();
            _OIDPerson = result.ToString();  
        }


        [Test(Description = "Modify in memory data within a transaction and rollback without resync on database")]
        public void InMemoryTransactionTest()
        {
            string personIOD = String.Empty;

            System.Transactions.CommittableTransaction tr = new System.Transactions.CommittableTransaction();
            try{
                Context.LocalStorage.Enlist(); //without that no chance  
                Personne result = CreatePersonne(Context);
                personIOD = result.ToString();
                tr.Rollback(); 
            }finally
            {
                tr.Rollback(); 
            }
           
            object obj = Context.Find<Personne>(personIOD);
            
            Assertion.AssertNull(obj);  
        }

        [Test(Description = "Modify in memory data within a transaction and rollback without resync on database")]
        public void InDBTransactionTest()
        {
            
            using (System.Transactions.TransactionScope scope = new System.Transactions.TransactionScope())
            {
                TestObjectdependencies();
                //explicitly forget the line bellow
                //scope.Complete(); 
            }

            object obj = Context.Find<Personne>(_OIDPerson);

            Assertion.AssertNull(obj);
        }

        [Test(Description = "attempt to clean a network of object, test dependancies detection")]
        public void TestClear()
        {
            TestObjectdependencies();

            Personne reloaded = Context.Find<Personne>(_OIDPerson);

            Context.LocalStorage.Delete(reloaded.CoordonneeSuccession);
            Context.LocalStorage.Delete(reloaded.Coordonnee);

            Context.LocalStorage.Delete(reloaded.Adresse.Lieu);
            Context.LocalStorage.Delete(reloaded.Adresse);

            Context.LocalStorage.Delete(reloaded.AdresseDeces.Lieu);
            Context.LocalStorage.Delete(reloaded.AdresseDeces);

            Context.LocalStorage.Delete(reloaded.AdresseNaissance.Lieu);
            Context.LocalStorage.Delete(reloaded.AdresseNaissance);

            Context.LocalStorage.Delete(reloaded.AdresseSuccession.Lieu);
            Context.LocalStorage.Delete(reloaded.AdresseSuccession);
            
            Context.Update();
        }

        [Test(Description="Create, reload, update, and Delete a Personne")]
        public void TestCyclePersonne()
        {
            
            Personne result = CreatePersonne(Context);
            string PersonOID = result.ToString(); 
            result.Nom = "Name " + result.GetHashCode();
            Context.Update();
            Personne reloaded = Context.Find<Personne>(PersonOID);
            reloaded.Nom += " reloaded";
            Context.Update();
            Personne reloadedTwice = Context.Find<Personne>(PersonOID);
            Context.LocalStorage.Delete(reloadedTwice);
            Context.Update();
            Personne final = Context.Find<Personne>(PersonOID);
            Assert(final == null); 
        }

        [Test]
        public void TestRole_Assure()
        {
            Role_Assure  result = CreateAssure(Context);
            result.PersonnePhysique.Nom= "Name " + result.GetHashCode();
            result.Login  = "Title " + result.GetHashCode();
            Context.Update();
        }

        [Test]
        public void TestProject()
        {
            Lieu result = CreateLieu(Context);
            result.Pays  = "Lieu Pays " + result.GetHashCode();
            Context.Update();
        }

        [Test]
        public void TestQuery()
        {
            System.Data.DataTable table = new System.Data.DataTable();
            Context.Fill(table, "Lieu");
        }

        //[Test]
        //public void TestPolymorphicQuery()
        //{
        //    DataTable table = new DataTable();
        //    Context.RemoteSource.Fill(table, "Personn");
        //}

        //[Test]
        //public void TestPolymorphicCollect()
        //{
        //    Employee employee = CreateEmployee(Context);
        //    employee.Name = "Employee " + employee.GetHashCode();

        //    Manager manager = CreateManager(Context);
        //    manager.Name = "Manager " + manager.GetHashCode();
        //    manager.Title = "Title " + manager.GetHashCode();

        //    Debug.Assert(Context.LocalStorage.GetCount("Employee") == 2);
        //    foreach (Employee e in Context.LocalStorage.Collect(typeof(Employee)))
        //    {
        //        Debug.Assert(Context.Domain.GetType(e) == typeof(Employee) || Context.Domain.GetType(e) == typeof(Manager));
        //    }
        //}

        [Test]
        public void TestLocalCollect()
        {
            Personne  employee = CreatePersonne(Context);
            employee.Nom = "Personne" + employee.GetHashCode();

            Role_Assure manager = CreateAssure(Context);
            manager.PersonnePhysique = employee;
            manager.Login = "login " + manager.GetHashCode();

            Debug.Assert(Context.LocalStorage.GetCount("Personne p where p.Nom like ?", "Personne%") == 1);
            foreach (Personne p in Context.LocalStorage.Collect("Personne p where p.Nom like ?", "Personne%"))
            {
                Debug.Assert(true);

                //Debug.Assert(Context.rootDomain.GetType(e) == typeof(Manager));
            }
        }

        [Test]
        public void TestJoin()
        {
            DataTable table = new DataTable();
            //Context.Fill(table, "select PersonnePhysique.Nom, Role_assure.Login from Role_assure , Role_assure.PersonnePhysique ");
            Context.Fill(table, "select pp.Nom, ra.Login from Role_assure ra, ra.PersonnePhysique pp ");
        }

    }
}
